<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_2/classes/prom_user.php $
 * $Id: prom_user.php 1519 2009-04-06 15:20:04Z quietust $
 */

if (!defined('IN_GAME'))
	die("Access denied");

require_once(PROM_BASEDIR .'classes/prom_entity.php');

class prom_user extends prom_entity
{
	// Constructor - initialize as User
	public function __construct($id = 0, $needlock = TRUE)
	{
		parent::__construct($id, $needlock, USER_TABLE, 'u_id', ENT_USER);
	}

	// Locates the user record having a particular username
	// Must be called using an uninitialized user object
	public function findName ($name)
	{
		if ($this->id != 0)
		{
			warning('Attempted to initialize already initialized entity of type '. get_class($this), 1);
			return FALSE;
		}

		global $db;

		$q = $db->prepare('SELECT u_id FROM '. USER_TABLE .' WHERE u_username = ?');
		db_bindStrValue($q, 1, $name);
		if (!$q->execute())
		{
			warning('Failed to determine ID for username', 1);
			return FALSE;
		}

		$id = $q->fetchColumn();
		if (!$id)
			return FALSE;
		$this->id = $id;
		return TRUE;
	}

	// Creates a brand new user record and inserts it into the database
	// Must be called using an uninitialized user object
	public function create ($username, $password, $name, $email)
	{
		if ($this->id != 0)
		{
			warning('Attempted to initialize already initialized entity of type '. get_class($this), 1);
			return FALSE;
		}

		global $db;

		// creation of the user and the lock should occur simultaneously to ensure the turns script doesn't steal the lock
		// this transaction will only work effectively if the user table is declared as InnoDB; as MyISAM, this may fail
		$db->beginTransaction();
		$q = $db->prepare('INSERT INTO '. USER_TABLE .' (u_username,u_email) VALUES (?,?)');
		db_bindStrValue($q, 1, $username);
		db_bindStrValue($q, 2, $email);
		if (!$q->execute())
		{
			warning('Failed to create user account', 1);
			return FALSE;
		}

		$this->id = $db->lastInsertId(db_getSequence($db, USER_TABLE));
		if (!$this->lock(LOCK_NEW))
		{
			// ...so if the turns script does happen to be running, it may snatch up the lock before we were able to take it
			warning('Unable to lock newly created entity '. $this->id .' of type '. get_class($this), 1);
			// we can't allow this to fail, so mark the entity as locked anyways - the turn script should leave it alone
			$this->load();
			$this->lockid = LOCK_NEW;
		}
		$db->commit();

		// set various properties of the new user
		$this->setPassword($password);
		$this->u_name = $name;
		$this->u_createdate = CUR_TIME;
		$this->u_lang = DEFAULT_LANGUAGE;
		$this->u_style = DEFAULT_STYLE;
		$this->u_timezone = DEFAULT_TIMEZONE;

		return TRUE;
	}

	// Sets user's password
	public function setPassword ($password)
	{
		if ($this->id == 0)
		{
			warning('Attempted to set password on uninitialized entity of type '. get_class($this), 2);
			return FALSE;
		}

		if (!$this->locked())
		{
			warning('Entity '. $this->id .' of type '. get_class($this) .' attempted to set password while not locked', 1);
			return FALSE;
		}

		$salt = md5(uniqid(mt_rand(), TRUE));
		$salt1 = substr($salt, 0, 12);
		$salt2 = substr($salt, -12);
		$password = $salt1 . sha1($salt1 . $password . $salt2) . $salt2;

		$this->u_password = $password;
		return TRUE;
	}

	// Verifies user's password
	public function checkPassword ($password)
	{
		if ($this->id == 0)
			return FALSE;

		$salt1 = substr($this->u_password, 0, 12);
		$salt2 = substr($this->u_password, -12);
		$password = $salt1 . sha1($salt1 . $password . $salt2) . $salt2;

		if ($this->u_password != $password)
			return FALSE;
		return TRUE;
	}

	// Shortcut for setting flags within "u_flags"
	public function setFlag ($flag)
	{
		return $this->setData('u_flags', $this->data['u_flags'] | $flag);
	}

	// Shortcut for clearing flags within "u_flags"
	public function clrFlag ($flag)
	{
		return $this->setData('u_flags', $this->data['u_flags'] & ~$flag);
	}

	// DEPRECATED - Formats a date in the user's timezone
	public function date ($format, $time)
	{
		return str_replace('+0000', sprintf('%+05d', $this->u_timezone / 36), gmdate($format, $time + $this->u_timezone));
	}

	// Formats a date using the user's configured date format
	public function customdate ($time)
	{
		$date = gmdate($this->u_dateformat, $time + $this->u_timezone);
		// The only timezone formats we can reliably substitute are 'O' and 'P' (and 'c' and 'r' by extension)
		// Anything else is by name ('e' and 'T') or wouldn't match reliably ('Z')
		$date = str_replace(array(
				'+0000',
				'+00:00',
			), array(
				sprintf('%+03d%02d', $this->u_timezone / 3600, $this->u_timezone % 60),
				sprintf('%+03d:%02d', $this->u_timezone / 3600, $this->u_timezone % 60),
			), $date);
		return $date;
	}

	// Formats user name+number as a string
	public function __toString ()
	{
		if ($this->id == 0)
			return lang('COMMON_USER_NAMEID', 'COMMON_USER_UNINITIALIZED', prenum(0));
		return lang('COMMON_USER_NAMEID', $this->u_name, prenum($this->u_id));
	}

	// Lock all user entities
	public static function lock_all ($owner)
	{
		parent::lock_all_entities(USER_TABLE, 'u_id', ENT_USER, $owner);
	}
} // class prom_user
?>
