<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_2/classes/prom_vars.php $
 * $Id: prom_vars.php 1519 2009-04-06 15:20:04Z quietust $
 */

if (!defined('IN_GAME'))
	die("Access denied");

require_once(PROM_BASEDIR .'classes/prom_entity.php');

class prom_vars extends prom_entity
{
	public function __construct($needlock = TRUE)
	{
		// variables table has special structure - no ID column
		// set ID to 1, and define dummy table/column names
		parent::__construct(1, $needlock, 'DUAL', 'NULL', ENT_VARS);
	}

	// Loads variables from the database
	public function load ()
	{
		global $db;

		$q = $db->query('SELECT * FROM '. VAR_TABLE);
		if (!$q)
		{
			warning('Entity '. $this->id .' of type '. get_class($this) .' failed to load from database', 1);
			return FALSE;
		}

		$data = $q->fetchAll();
		foreach ($data as $var)
			$this->data[$var['v_name']] = $var['v_value'];

		$this->update = array();
		$this->lockid = 0;
		return TRUE;
	}

	// Creates a new world variable and inserts it into the database
	public function create ($name)
	{
		if (!$this->locked())
		{
			warning('Entity '. $this->id .' of type '. get_class($this) .' attempted to create new variable while not locked', 1);
			return FALSE;
		}

		global $db;

		$q = $db->prepare('INSERT INTO '. VAR_TABLE .' (v_name) VALUES (?)');
		db_bindStrValue($q, 1, $name);
		if (!$q->execute())
		{
			warning('Failed to create new world variable "'. $name .'"', 1);
			return FALSE;
		}

		$this->data[$name] = '';
		return TRUE;
	}

	// Enqueues an adjustment for a numeric world variable
	public function adjust ($name, $offset)
	{
		global $db;

		$q = $db->prepare('INSERT INTO '. VAR_ADJUST_TABLE .' (v_name,v_offset) VALUES (?,?)');
		db_bindStrValue($q, 1, $name);
		db_bindIntValue($q, 2, $offset);
		if (!$q->execute())
		{
			warning('Failed to enqueue adjustment for world variable "'. $name .'"', 1);
			return FALSE;
		}
		return TRUE;
	}

	// Saves world variables back to the database
	// Specify FALSE to keep it locked afterwards
	public function save ($unlock = TRUE)
	{
		if (!$this->locked())
		{
			warning('Entity '. $this->id .' of type '. get_class($this) .' attempted to save while not locked', 1);
			return FALSE;
		}
		// if there are no fields to update, simply perform an unlock
		if (count($this->update) == 0)
			return $unlock ? $this->unlock() : TRUE;
		$this->update = array_unique($this->update);

		global $db;

		$q = $db->prepare('UPDATE '. VAR_TABLE .' SET v_value = ? WHERE v_name = ?');
		foreach ($this->update as $field)
		{
			db_bindStrValue($q, 2, $field);
			if (is_numeric($this->data[$field]))
				db_bindIntValue($q, 1, $this->data[$field]);
			else	db_bindStrValue($q, 1, $this->data[$field]);
			if (!$q->execute())
			{
				// if even a single variable fails to save, then return failure
				warning('Entity '. $this->id .' of type '. get_class($this) .' failed to save variable "'. $field .'" to database', 1);
				return FALSE;
			}
		}

		$this->update = array();
		return $unlock ? $this->unlock() : TRUE;
	}
} // class prom_vars
?>
