<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_2/pages/admin/permissions.php $
 * $Id: permissions.php 1568 2009-04-16 16:54:06Z quietust $
 */

if (!defined('IN_GAME'))
	die("Access denied");

$title = 'ADMIN_PERMISSIONS_TITLE';

$needpriv = UFLAG_ADMIN;

page_header();

$permlabels = array(PERM_IPV4 => lang('ADMIN_PERMISSIONS_TYPE_IPV4'), PERM_IPV6 => lang('ADMIN_PERMISSIONS_TYPE_IPV6'), PERM_EMAIL => lang('ADMIN_PERMISSIONS_TYPE_EMAIL'));

if ($action == 'add') do
{
	$indata = getFormVar('perm_data');
	$duration = fixInputNum(getFormVar('perm_duration'));
	$type = getFormVar('perm_type');
	$except = fixInputBool(getFormVar('perm_except'));
	$comment = htmlspecialchars(getFormVar('perm_comment'));
	$reason = htmlspecialchars(getFormVar('perm_reason'));

	if (!isset($permlabels[$type]))
	{
		notice(lang('ADMIN_PERMISSIONS_INVALID_TYPE'));
		break;
	}
	if (strlen($comment) > 255)
	{
		notice(lang('ADMIN_PERMISSIONS_COMMENT_TOO_LONG'));
		break;
	}
	if (lang_isset($comment))
	{
		notice(lang('ADMIN_PERMISSIONS_COMMENT_INVALID'));
		break;
	}
	if (strlen($reason) > 255)
	{
		notice(lang('ADMIN_PERMISSIONS_REASON_TOO_LONG'));
		break;
	}
	if (lang_isset($reason))
	{
		notice(lang('ADMIN_PERMISSIONS_REASON_INVALID'));
		break;
	}

	if ($type == PERM_IPV4)
	{
		// Straight IP address: "192.168.1.1"
		if (preg_match('/^(?:\d{1,3}\.){3}\d{1,3}$/', $indata))
		{
			$addr = $indata;
			$mask = '255.255.255.255';
		}
		// Wildcard class C: "192.168.1.*"
		elseif (preg_match('/^(?:\d{1,3}\.){3}\*$/', $indata))
		{
			$addr = str_replace('*', '0', $indata);
			$mask = '255.255.255.0';
		}
		// Wildcard class B: "192.168.*.*"
		elseif (preg_match('/^(?:\d{1,3}\.){3}\*\.\*$/', $indata))
		{
			$addr = str_replace('*', '0', $indata);
			$mask = '255.255.0.0';
		}
		// Wildcard class A: "192.*.*.*"
		elseif (preg_match('/^(?:\d{1,3}\.){3}\*\.\*\.\*$/', $indata))
		{
			$addr = str_replace('*', '0', $indata);
			$mask = '255.0.0.0';
		}
		// CIDR notation: "192.168.1.0/24"
		elseif (preg_match('/^(?:\d{1,3}\.){3}\d{1,3}\/\d{1,2}$/', $indata))
		{
			list($addr, $cidr) = explode('/', $indata);
			if (($cidr < 1) || ($cidr > 32))
			{
				notice(lang('ADMIN_PERMISSIONS_INVALID_DATA'));
				break;
			}
			// generate mask
			$mask = long2ip(ip2long('255.255.255.255') << (32 - $cidr));
			// and mask off the address
			$addr = long2ip(ip2long($addr) & ip2long($mask));
		}
		// Address/mask notation: "192.168.1.0/255.255.255.0"
		elseif (preg_match('/^(?:\d{1,3}\.){3}\d{1,3}\/(?:\d{1,3}\.){3}\d{1,3}$/', $indata))
		{
			list($addr, $mask) = explode('/', $indata);
			// mask off the address
			$addr = long2ip(ip2long($addr) & ip2long($mask));
		}
		else
		{
			notice(lang('ADMIN_PERMISSIONS_INVALID_DATA'));
			break;
		}
		$outdata = $addr.'/'.$mask;
	}
	if ($type == PERM_IPV6)
	{
		// Add CIDR suffix if not present - all formats require it
		$addr = explode('/', $indata);
		if (count($addr) == 1)
			$cidr = 128;
		else	$cidr = $addr[1];
		$addr = $addr[0];
		if (($cidr < 1) || ($cidr > 128))
		{
			notice(lang('ADMIN_PERMISSIONS_INVALID_DATA'));
			break;
		}
		// Regular expressions borrowed from Net-IPv6Addr-0.2
		// Full IP address
		if (preg_match('/^(?:[0-9A-F]{1,4}:){7}[0-9A-F]{1,4}$/i', $addr))
			;
		// Partial addresses
		elseif (preg_match('/^[0-9A-F]{0,4}::$/i', $addr))
			;
		elseif (preg_match('/^:(?::[0-9A-F]{1,4}){1,6}$/i', $addr))
			;
		elseif (preg_match('/^(?:[0-9A-F]{1,4}:){1,6}:$/i', $addr))
			;
		elseif (preg_match('/^(?:[0-9A-F]{1,4}:)(?::[0-9A-F]{1,4}){1,6}$/i', $addr))
			;
		elseif (preg_match('/^(?:[0-9A-F]{1,4}:){2}(?::[0-9A-F]{1,4}){1,5}$/i', $addr))
			;
		elseif (preg_match('/^(?:[0-9A-F]{1,4}:){3}(?::[0-9A-F]{1,4}){1,4}$/i', $addr))
			;
		elseif (preg_match('/^(?:[0-9A-F]{1,4}:){4}(?::[0-9A-F]{1,4}){1,3}$/i', $addr))
			;
		elseif (preg_match('/^(?:[0-9A-F]{1,4}:){5}(?::[0-9A-F]{1,4}){1,2}$/i', $addr))
			;
		elseif (preg_match('/^(?:[0-9A-F]{1,4}:){6}(?::[0-9A-F]{1,4})$/i', $addr))
			;
		// IPv4 compatibility addresses
		elseif (preg_match('/^(?:0:){5}ffff:(?:\d{1,3}\.){3}\d{1,3}$/i', $addr))
			;
		elseif (preg_match('/^(?:0:){6}(?:\d{1,3}\.){3}\d{1,3}$/i', $addr))
			;
		elseif (preg_match('/^::(?:ffff:)?(?:\d{1,3}\.){3}\d{1,3}$/i', $addr))
			;
		else
		{
			notice(lang('ADMIN_PERMISSIONS_INVALID_DATA'));
			break;
		}
		$outdata = $addr.'/'.$cidr;
	}
	if ($type == PERM_EMAIL)
	{
		// validate the email address with wildcards replaced with innocuous text
		if (!validate_email(str_replace('*', 'a', $indata)))
		{
			notice(lang('ADMIN_PERMISSIONS_INVALID_DATA'));
			break;
		}
		$outdata = $indata;
	}

	if ($except)
	{
		$type |= PERM_EXCEPT;
		$reason = '';
	}

	if ($duration == 0)
		$expire = 0;
	else	$expire = $duration * 86400 + CUR_TIME;

	$q = $db->prepare('INSERT INTO '. PERMISSION_TABLE .' (p_type,p_criteria,p_comment,p_reason,p_timestamp,p_expire) VALUES (?,?,?,?,?,?)');
	db_bindIntValue($q, 1, $type);
	db_bindIntValue($q, 2, $outdata);
	db_bindStrValue($q, 3, $comment);
	db_bindStrValue($q, 4, $reason);
	db_bindStrValue($q, 5, CUR_TIME);
	db_bindStrValue($q, 6, $expire);
	$q->execute() or warning('Failed to add record to permissions table', 0);
	notice(lang('ADMIN_PERMISSIONS_COMPLETE'));
	logevent(varlist(array('type', 'indata', 'duration', 'comment', 'reason'), get_defined_vars()));
} while (0);
if ($action == 'remove') do
{
	$id = fixInputNum(getFormVar('perm_id'));

	$q = $db->prepare('DELETE FROM '. PERMISSION_TABLE .' WHERE p_id = ?');
	db_bindIntValue($q, 1, $id);
	$q->execute() or warning('Failed to delete record from permissions table', 0);
	notice(lang('ADMIN_PERMISSIONS_COMPLETE'));
	logevent(varlist(array('id'), get_defined_vars()));
} while (0);
notices();

$q = $db->query('SELECT * FROM '. PERMISSION_TABLE .' ORDER BY p_type ASC, p_criteria ASC') or warning('Failed to read permission table', 0);
$permissions = $q->fetchAll();
?>
<table>
<tr><th><?=lang('ADMIN_PERMISSIONS_COLUMN_ACTION')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_TYPE')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_CRITERIA')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_DATE')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_LASTHIT')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_HITCOUNT')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_EXPIRE')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_COMMENT')?></th>
    <th><?=lang('ADMIN_PERMISSIONS_COLUMN_REASON')?></th></tr>
<?
foreach ($permissions as $record)
{
?>
<tr><td class="ac"><form method="post" action="?location=admin/permissions"><input type="hidden" name="action" value="remove" /><input type="hidden" name="perm_id" value="<?=$record['p_id']?>" /><input type="submit" value="<?=lang('ADMIN_PERMISSIONS_REMOVE_SUBMIT')?>" /></form></td>
    <td><?if ($record['p_type'] & PERM_EXCEPT) echo lang('ADMIN_PERMISSIONS_FORMAT_ALLOW', $permlabels[$record['p_type'] & PERM_MASK]); else echo lang('ADMIN_PERMISSIONS_FORMAT_DENY', $permlabels[$record['p_type'] & PERM_MASK]); ?></td>
    <td><?=$record['p_criteria']?></td>
    <td><?=$user1->customdate($record['p_timestamp'])?></td>
    <td><?if ($record['p_type'] & PERM_EXCEPT) echo lang('ADMIN_PERMISSIONS_UNUSED'); elseif ($record['p_lasthit'] == 0) echo lang('ADMIN_PERMISSIONS_DATE_NEVER'); else echo $user1->customdate($record['p_lasthit']);?></td>
    <td><?if ($record['p_type'] & PERM_EXCEPT) echo lang('ADMIN_PERMISSIONS_UNUSED'); else echo $record['p_hitcount'];?></td>
    <td><?if ($record['p_expire'] == 0) echo lang('ADMIN_PERMISSIONS_DATE_FOREVER'); else echo $user1->customdate($record['p_expire']);?></td>
    <td><?=$record['p_comment']?></td>
    <td><?if ($record['p_type'] & PERM_EXCEPT) echo lang('ADMIN_PERMISSIONS_UNUSED'); else echo $record['p_reason'];?></td></tr>
<?
}
?>
</table>
<form method="post" action="?location=admin/permissions">
<table class="inputtable">
<tr><th><?=lang('ADMIN_PERMISSIONS_LABEL_TYPE')?></th><td><?=radiobutton('perm_type', $permlabels[PERM_IPV4], PERM_IPV4, TRUE)?> <?=radiobutton('perm_type', $permlabels[PERM_IPV6], PERM_IPV6)?> <?=radiobutton('perm_type', $permlabels[PERM_EMAIL], PERM_EMAIL)?><br /><?=checkbox('perm_except', lang('ADMIN_PERMISSIONS_TYPE_EXCEPT'))?></td></tr>
<tr><th><?=lang('ADMIN_PERMISSIONS_LABEL_CRITERIA')?></th><td><input type="text" name="perm_data" /></td></tr>
<tr><th><?=lang('ADMIN_PERMISSIONS_LABEL_COMMENT')?></th><td><input type="text" name="perm_comment" maxlength="255" /><br /><?=lang('ADMIN_PERMISSIONS_LABEL_COMMENT_NOTE')?></td></tr>
<tr><th><?=lang('ADMIN_PERMISSIONS_LABEL_REASON')?></th><td><input type="text" name="perm_reason" maxlength="255" /><br /><?=lang('ADMIN_PERMISSIONS_LABEL_REASON_NOTE')?></td></tr>
<tr><th><?=lang('ADMIN_PERMISSIONS_LABEL_DURATION')?></th><td><input type="text" name="perm_duration" size="4" value="0" /><br /><?=lang('ADMIN_PERMISSIONS_LABEL_DURATION_NOTE')?></td></tr>
<tr><td colspan="2" class="ac"><input type="hidden" name="action" value="add" /><input type="submit" value="<?=lang('ADMIN_PERMISSIONS_ADD_SUBMIT')?>" /></td></tr>
</table>
</form>
<?
page_footer();
?>
