<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_2/pages/admin/round.php $
 * $Id: round.php 1584 2009-04-30 13:09:18Z quietust $
 */

if (!defined('IN_GAME'))
	die("Access denied");

$title = 'ADMIN_ROUND_TITLE';

$needpriv = UFLAG_ADMIN;

if ($action == 'update')
	$lock['world'] = -1;

page_header();

if ($action == 'update') do
{
	$begin = getFormVar('round_begin');
	$closing = getFormVar('round_closing');
	$end = getFormVar('round_end');

	$_begin = strtotime($begin);
	$_closing = strtotime($closing);
	$_end = strtotime($end);

	if ($_begin == FALSE)
	{
		notice(lang('SETUP_BAD_START'));
		break;
	}
	if ($_closing == FALSE)
	{
		notice(lang('SETUP_BAD_COOLDOWN'));
		break;
	}
	if ($_end == FALSE)
	{
		notice(lang('SETUP_BAD_END'));
		break;
	}

	if ($_begin >= $_closing)
	{
		notice(lang('SETUP_COOLDOWN_AFTER_START'));
		break;
	}

	if ($_closing + (VACATION_START + VACATION_LIMIT) * 3600 >= $_end)
	{
		notice(lang('SETUP_COOLDOWN_LENGTH', ceil((VACATION_START + VACATION_LIMIT) / 24)));
		break;
	}

	$world->round_time_begin = $_begin;
	$world->round_time_closing = $_closing;
	$world->round_time_end = $_end;

	notice(lang('ADMIN_ROUND_DATES_UPDATED'));
	logevent(varlist(array('begin', 'closing', 'end'), get_defined_vars()));

	// if the reset checkbox isn't selected, just skip the rest
	$doreset = fixInputBool(getFormVar('round_reset'));
	if (!$doreset)
		break;

	$confirm = fixInputBool(getFormVar('reset_confirm'));
	if (!$confirm)
	{
		notice(lang('ADMIN_ROUND_RESTART_NEED_CONFIRM'));
		break;
	}
	$list = getFormArr('reset_acctlist');
	$newemps = array();
	foreach ($list as $id)
	{
		$name = htmlspecialchars(getFormVar('reset_empname_'. $id));
		if (!strlen($name))
		{
			notice(lang('ADMIN_ROUND_RESTART_NEED_NAMES'));
			break 2;
		}
		$newemps[$id] = $name;
	}
	if (count($newemps) == 0)
	{
		notice(lang('ADMIN_ROUND_RESTART_NEED_EMPIRES'));
		break;
	}

	// Release lock on current empire so we can lock everything below
	// The only other entity that should've been locked was $world
	$emp1->save();

	// Lock everything in the database, so we can delete things without bad stuff happening
	// Since we've locked world variables, this can't collide with the turns script

	// First is users - these will be staying behind, but we need to clear out their sessions
	prom_user::lock_all(LOCK_RESET);

	// Next is empires - these need to be deleted
	prom_empire::lock_all(LOCK_RESET);

	if (CLAN_ENABLE)
	{
		// And now clans - also to be deleted
		prom_clan::lock_all(LOCK_RESET);
	}

	// empty out all database tables in an appropriate order
	// always clear out clans, even if they're not enabled
	if (!db_clearTable($db, SESSION_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', SESSION_TABLE));
		break;
	}
	if (!db_clearTable($db, EMPIRE_NEWS_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', EMPIRE_NEWS_TABLE));
		break;
	}
	if (!db_clearTable($db, CLAN_INVITE_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', CLAN_INVITE_TABLE));
		break;
	}
	if (!db_clearTable($db, CLAN_RELATION_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', CLAN_RELATION_TABLE));
		break;
	}
	if (!db_clearTable($db, CLAN_MESSAGE_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', CLAN_MESSAGE_TABLE));
		break;
	}
	if (!db_clearTable($db, CLAN_TOPIC_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', CLAN_TOPIC_TABLE));
		break;
	}
	if (!db_clearTable($db, CLAN_NEWS_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', CLAN_NEWS_TABLE));
		break;
	}
	if (!db_clearTable($db, CLAN_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', CLAN_TABLE));
		break;
	}
	if (!db_clearTable($db, LOTTERY_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', LOTTERY_TABLE));
		break;
	}
	if (!db_clearTable($db, MARKET_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', MARKET_TABLE));
		break;
	}
	if (!db_clearTable($db, EMPIRE_MESSAGE_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', EMPIRE_MESSAGE_TABLE));
		break;
	}
	if (!db_clearTable($db, EMPIRE_EFFECT_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', EMPIRE_EFFECT_TABLE));
		break;
	}
	if (!db_clearTable($db, EMPIRE_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', EMPIRE_TABLE));
		break;
	}
	if (!db_clearTable($db, VAR_ADJUST_TABLE))
	{
		notice(lang('ADMIN_ROUND_RESTART_ERROR_CLEAR_TABLE', VAR_ADJUST_TABLE));
		break;
	}

	// release all locks on the now-deleted empires and clans
	$q = $db->prepare('DELETE FROM '. LOCK_TABLE .' WHERE lock_owner = ? AND lock_type IN (?,?)');
	db_bindIntValue($q, 1, LOCK_RESET);
	db_bindIntValue($q, 2, ENT_EMPIRE);
	db_bindIntValue($q, 3, ENT_CLAN);
	$q->execute() or warning('Failed to release database locks during reset', 0);

	// reinitialize all appropriate world variables
	$world->lotto_current_jackpot = LOTTERY_JACKPOT;
	$world->lotto_yesterday_jackpot = LOTTERY_JACKPOT;
	$world->lotto_last_picked = 0;
	$world->lotto_last_winner = 0;
	$world->lotto_jackpot_increase = 0;

	// reset empire stored in session
	$_SESSION['empire'] = $want_emp = 0;
	// create new empires
	foreach ($newemps as $num => $name)
	{
		$user = new prom_user($num, FALSE);
		$user->lock(LOCK_RESET);
		$emp = new prom_empire();
		$emp->create($user, $name, 1);
		// if we're creating a new empire for your account, then bind the session to it
		if ($user->u_id == $user1->u_id)
			$_SESSION['empire'] = $emp->e_id;
		else	$want_emp = $emp->e_id;
		// copy privileges from user account, and pre-mark them as validated
		if ($user->u_flags & (UFLAG_ADMIN | UFLAG_MOD))
			$emp->setFlag(EFLAG_ADMIN);
		$emp->setFlag(EFLAG_VALID);
		$emp->save();
		$user->save();
	}
	// if you didn't make a new empire for yourself, set-user yourself to the last account anyways
	// you'll be logging out shortly anyways, so it works out
	if (!$_SESSION['empire'])
		$_SESSION['empire'] = $want_emp;

	// finally, release all locks on users
	$q = $db->prepare('DELETE FROM '. LOCK_TABLE .' WHERE lock_owner = ? AND lock_type = ?');
	db_bindIntValue($q, 1, LOCK_RESET);
	db_bindIntValue($q, 2, ENT_USER);
	$q->execute() or warning('Failed to release database locks following reset', 0);
	notice(lang('ADMIN_ROUND_RESTART_COMPLETE'));
	logevent();
} while (0);

notices();
?>
<form method="post" action="?location=admin/round">
<table class="inputtable">
<tr><th class="ar"><?=lang('LABEL_ROUND_START')?></th>
    <td><input type="text" name="round_begin" value="<?=gmdate('Y/m/d H:i:s O', $world->round_time_begin)?>" size="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_ROUND_COOLDOWN')?></th>
    <td><input type="text" name="round_closing" value="<?=gmdate('Y/m/d H:i:s O', $world->round_time_closing)?>" size="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_ROUND_END')?></th>
    <td><input type="text" name="round_end" value="<?=gmdate('Y/m/d H:i:s O', $world->round_time_end)?>" size="24" /></td></tr>
</table>
<table>
<tr><td colspan="6" class="ac"><?=checkbox('round_reset', lang('ADMIN_ROUND_RESTART'))?></td></tr>
<tr><th><?=lang('COLUMN_ADMIN_USERID')?></th>
    <th><?=lang('COLUMN_ADMIN_NICKNAME')?></th>
    <th><?=lang('COLUMN_ADMIN_USERNAME')?></th>
    <th><?=lang('COLUMN_ADMIN_EMAIL')?></th>
    <th><?=lang('COLUMN_ADMIN_FLAGS')?></th>
    <th><?=lang('ADMIN_ROUND_RESTART_CREATE_EMPIRE')?></th></tr>
<?
$q = $db->prepare('SELECT u_id,u_name,u_username,u_email,u_flags FROM '. USER_TABLE .' WHERE u_flags & ? != 0 ORDER BY u_id ASC');
db_bindIntValue($q, 1, UFLAG_ADMIN | UFLAG_MOD);
$q->execute() or warning('Failed to fetch privileged user list', 0);
$users = $q->fetchAll();
foreach ($users as $user)
{
?>
<tr><th class="ar"><?=$user['u_id']?></th>
    <td class="al"><?=$user['u_name']?></td>
    <td class="ar"><?=htmlspecialchars($user['u_username'])?></td>
    <td class="ac"><?=$user['u_email']?></td>
    <td class="ac"><?
	echo ($user['u_flags'] & UFLAG_ADMIN) ? 'A' : '-';
	echo ($user['u_flags'] & UFLAG_MOD) ? 'M' : '-';
	echo ($user['u_flags'] & UFLAG_DISABLE) ? 'D' : '-';
	echo ($user['u_flags'] & UFLAG_VALID) ? 'V' : '-';
?></td>
    <td><?=checkbox('reset_acctlist[]', '', $user['u_id'])?> <input type="text" name="reset_empname_<?=$user['u_id']?>" value="" /></td></tr>
<?
}
?>
<tr><td colspan="5"><?=checkbox('reset_confirm', lang('ADMIN_ROUND_RESTART_CONFIRM'))?></td>
    <td><input type="hidden" name="action" value="update" /><input type="submit" value="<?=lang('ADMIN_ROUND_SUBMIT')?>" /></td></tr>
<tr><td colspan="6" class="ac"><b><?=lang('ADMIN_ROUND_RESTART_REMIND_HISTORY')?></b></td></tr>
</table>
</form>
<?
page_footer();
?>
